package ch.epfl.ppo;

import java.util.Iterator;
import java.util.NoSuchElementException;

public abstract class FilteringIterator<E> extends ReadOnlyIterator<E> {
    private final Iterator<E> underlyingIt;
    private E next;
    private boolean isNextValid;

    public FilteringIterator(Iterator<E> underlyingIt) {
        this.underlyingIt = underlyingIt;
        this.next = null;
        this.isNextValid = false;
    }

    public abstract boolean shouldInclude(E elem);

    @Override
    public boolean hasNext() {
        while (!isNextValid && underlyingIt.hasNext()) {
            next = underlyingIt.next();
            isNextValid = shouldInclude(next);
        }
        return isNextValid;
    }

    @Override
    public E next() {
        if (! hasNext())
            throw new NoSuchElementException();
        isNextValid = false;
        return next;
    }
}
